import REST from '../../api/rest-axios';
import { fetchFromRelationships, getCleanDate } from './contentFetchUtils';


export async function getCarteSensible(partie) {
    const carteSensiblePromise = fetchFromRelationships('field_image_carte', partie.relationships);

    const carteSensibleData = await carteSensiblePromise;
    if (carteSensibleData) {
        return {
            url: {
                original: carteSensibleData.attributes.uri.url,
                small: carteSensibleData.attributes.image_style_uri.content_small,
                medium: carteSensibleData.attributes.image_style_uri.content_medium,
                large: carteSensibleData.attributes.image_style_uri.content_large,
                xlarge: carteSensibleData.attributes.image_style_uri.content_x_large,
            },
            alt: partie.relationships.field_image_carte.data.meta.alt,
        };
    }
}

export async function getTitreTexte(partie) {
    let titre = partie.attributes.field_titre;
    let texte = partie.attributes.field_texte.value;                                    
    
    // get the resized images from the text
    const imgRegex = /<img[^>]+>/g;
    const uuidRegex = /data-entity-uuid="([^"]+)"/;
    const imgTags = texte.match(imgRegex);
    
    if (imgTags) {
        const imagePromises = imgTags.map(imgTag => {
            const uuidMatch = imgTag.match(uuidRegex);
            if (uuidMatch && uuidMatch[1]) {
                return REST.get(`/jsonapi/file/file/${uuidMatch[1]}`)
                    .then(response => ({
                        originalTag: imgTag,
                        imageData: response.data.data
                    }));
            }
        });

        const images = await Promise.all(imagePromises);
        images.forEach(({originalTag, imageData}) => {
            const newImgTag = originalTag
                .replace(/src="[^"]+"/,`src="${imageData.attributes.image_style_uri.content_medium}"`)
                .replace('>',' data-large-src="' + imageData.attributes.image_style_uri.content_large + '">');
            texte = texte.replace(originalTag, newImgTag);
        });
    }

    return { titre, texte };
}

export async function getChiffresCles(partie) {
    const chiffresClesFetch = await fetchFromRelationships('field_chiffres_clefs', partie.relationships);
    if (chiffresClesFetch) {
        let chiffresCles = [];
        for (let chiffre of chiffresClesFetch) {
            chiffresCles.push({
                chiffre: chiffre.attributes.field_chiffre,
                description: chiffre.attributes.field_description,
            });
        }
        return chiffresCles;
    }    
}

export async function getDiaporama(partie) {
    const diaporamaFetch = await fetchFromRelationships('field_diaporama', partie.relationships);
    
    if (diaporamaFetch) {
        const diaporamaPromises = diaporamaFetch.map((image, index) => {
            return {
                url: {
                    original: image.attributes.uri.url,
                    small: image.attributes.image_style_uri.content_small,
                    medium: image.attributes.image_style_uri.content_medium,
                    large: image.attributes.image_style_uri.content_large,
                },
                alt: partie.relationships.field_diaporama.data[index].meta.alt,
            };
        });

        return await Promise.all(diaporamaPromises);
    }
}

export async function getEntretien(partie) {
    const [personnesFetch, questionsReponsesFetch] = await Promise.all([
        fetchFromRelationships('field_personne_s', partie.relationships),
        fetchFromRelationships('field_questions_reponses', partie.relationships)
    ]);

    if (personnesFetch && questionsReponsesFetch) {
        const personnesPromises = personnesFetch.map(async (personne) => {
            const portraitFetch = await fetchFromRelationships('field_portrait', personne.relationships);
            if (portraitFetch) {
                return {
                    portrait: {
                        original: portraitFetch.attributes.uri.url,
                        small: portraitFetch.attributes.image_style_uri.content_small,
                        medium: portraitFetch.attributes.image_style_uri.content_medium,
                        large: portraitFetch.attributes.image_style_uri.content_large,
                    },
                    alt: personne.relationships.field_portrait.data.meta.alt,
                    description: personne.attributes.field_description,
                };
            }
        });

        const questionsReponses = questionsReponsesFetch.map(qr => ({
            question: qr.attributes.field_question,
            reponse: qr.attributes.field_reponse.value,
        }));

        return {
            titre: partie.attributes.field_titre,
            personnes: await Promise.all(personnesPromises),
            questionsReponses: questionsReponses
        }
    }
}

export function getVideos(partie) {
    let videos = [];
    for (let video of partie.attributes.field_videos) {
        const videoId = video.split('?v=')[1];
        const videoUrl = `https://www.youtube.com/embed/${videoId}`;
        videos.push(videoUrl);
    }
    return videos;
}

export async function getDocument(partie) {
    // const documentFetch = await fetchFromRelationships('field_document', partie.relationships);
    const uuid = partie.relationships.field_document.data.id;
    const documentFetch = await REST.get(`/jsonapi/file/file/${uuid}`);
    const url = documentFetch.data.data.attributes.uri.url;

    const titre = partie.attributes.field_titre;
    const sousTitre = partie.attributes.field_sous_titre;
    const date = partie.attributes.field_date ? getCleanDate(partie.attributes.field_date) : null;
    const auteurice = partie.relationships.field_autheurice_s;
    const description = partie.relationships.field_document.data.meta.description;

    const vignetteFetch = await REST.get(`/jsonapi/file/file/${partie.relationships.field_vignette.data.id}`);
    const vignette = { url: vignetteFetch.data.data.attributes.image_style_uri.content_small, alt: partie.relationships.field_vignette.data.meta.alt };
    
    return { url, titre, sousTitre, date, auteurice, description, vignette };
}

export async function getGallerie(partie) {
    const gallerieFetch = await fetchFromRelationships('field_gallerie', partie.relationships);

    if (gallerieFetch) {
        const titre = gallerieFetch.attributes.title;
        const introduction = gallerieFetch.attributes.body?.processed;
        
        const imagesFetch = await fetchFromRelationships('field_images', gallerieFetch.relationships);
        let images = [];
        imagesFetch.forEach((image, index) => {
            images.push({
                url: {
                    original: image.attributes.uri.url,
                    small: image.attributes.image_style_uri.content_small,
                    medium: image.attributes.image_style_uri.content_medium,
                    large: image.attributes.image_style_uri.content_large,
                },
                alt: gallerieFetch.relationships.field_images.data[index].meta.alt,
            });
        });
        return { titre, introduction, images };
    }    
}