import REST from '../../api/rest-axios';

export async function fetchFromRelationships(field, relationships) {
    field = relationships[field] ? field : `${field}_static`;
    
    if (relationships[field]?.links) {
        const contentLink = relationships[field].links.related.href;
        return REST.get(contentLink)
            .then(contentFetch => contentFetch.data.data)
            .catch(error => {
                this.error = 'Failed to fetch data';
                console.error('Issue with getNodeData', error);
            });
    }
    return null;
}

export function getCleanDate(date) {
    return {
      d: date.split('-')[2],
      m: new Intl.DateTimeFormat('fr-FR', { month: 'long' }).format(new Date(date)),
      y: date.split('-')[0],
    }
}

export async function getRelatedEtape(direction, path) {
    const getRelatedEtapeContent = (relatedEtapeData) => {
        if (relatedEtapeData) {
            return fetchFromRelationships('field_vignette', relatedEtapeData.relationships)
                .then(vignetteFetch => {
                    if (vignetteFetch) {
                        return {
                            url: relatedEtapeData.attributes.metatag.find(tag => tag.tag === "link")?.attributes.href,
                            couleur: relatedEtapeData.attributes.field_couleur,
                            title: relatedEtapeData.attributes.title,
                            postalCode: relatedEtapeData.attributes.field_adresse.postal_code,
                            dates: {
                                start: getCleanDate(relatedEtapeData.attributes.field_dates.value),
                                end: getCleanDate(relatedEtapeData.attributes.field_dates.end_value),
                            },
                            vignette: {
                                url: {
                                    original: vignetteFetch.attributes.uri.url,
                                    small: vignetteFetch.attributes.image_style_uri.content_small,
                                    medium: vignetteFetch.attributes.image_style_uri.content_medium,
                                    large: vignetteFetch.attributes.image_style_uri.content_large,
                                },
                                alt: relatedEtapeData.relationships.field_vignette.data.meta.alt,
                            },
                        }
                    }
                });
        }
    }

    const allEtapesPromise = REST.get('/jsonapi/views/etapes/block_1/');
    
    return allEtapesPromise.then(allEtapesData => {
        for (let [index, etape] of allEtapesData.data.data.entries()) {
            if (etape.attributes.metatag.some(tag => 
                tag.tag === "link" && tag.attributes.href === path
            )) {
                const relatedEtapeIndex = direction ? (direction === 'next' ? index + 1 : index - 1) : index;
                return getRelatedEtapeContent(allEtapesData.data.data[relatedEtapeIndex]);
            }
        }
    });    
}

export async function getRessourceItemCard(item) {    
    try {
        const ressourceFetch = await REST.get(item.links.self.href);
        
        const partiesFetch = await REST.get(item.relationships.field_parties_ressource.links.related.href);
        const parties = partiesFetch.data.data;
        
        const vignettePartie = parties.find(partie => partie.type !== "paragraph--titre_texte");
        
        let vignette = null;
        if (vignettePartie) {
            let alt;
            switch (vignettePartie.type) {
                case 'paragraph--diaporama':
                    alt = vignettePartie.relationships.field_diaporama.data[0].meta.alt;
                    const diaporamaFetch = await REST.get(vignettePartie.relationships.field_diaporama.links.related.href);
                    vignette = {
                        url: diaporamaFetch.data.data[0].attributes.image_style_uri.content_small,
                        alt
                    };
                    break;
                
                case 'paragraph--video':
                    const videoId = vignettePartie.attributes.field_videos[0].split('?v=')[1];
                    vignette = {
                        url: `https://img.youtube.com/vi/${videoId}/0.jpg`,
                        alt: item.attributes.title
                    };
                    break;
                
                case 'paragraph--galleries':
                    const gallerieFetch = await REST.get(vignettePartie.relationships.field_gallerie.links.related.href);
                    const galleryAlt = gallerieFetch.data.data.relationships.field_images.data[0].meta.alt;
                    const gallerieImageFetch = await REST.get(gallerieFetch.data.data.relationships.field_images.links.related.href);
                    vignette = {
                        url: gallerieImageFetch.data.data[0].attributes.image_style_uri.content_small,
                        alt: galleryAlt
                    };
                    break;
                
                case 'paragraph--document':
                    alt = vignettePartie.relationships.field_vignette.data.meta.alt;
                    const documentFetch = await REST.get(vignettePartie.relationships.field_vignette.links.related.href);
                    vignette = {
                        url: documentFetch.data.data.attributes.image_style_uri.content_small,
                        alt
                    };
                    break;
                
                default:
                    vignette = null;
            }
        }
        
        return {
            ressourceType: item.attributes.field_type_de_ressource,
            title: item.attributes.title,
            auteurice: item.attributes.field_autheurice,
            date: getCleanDate(item.attributes.field_date_ressource),
            url: ressourceFetch.data.data.attributes.metatag.find(tag => tag.tag === "link")?.attributes.href,
            vignette
        };
    } catch (error) {
        console.error('Error fetching resource:', error);
        return null;
    }
}

export async function getRelatedRessources(etapeId) {
    const ressources = await REST.get(`/jsonapi/node/ressource/`);
    const ressourcesRelatedToEtape = ressources.data.data.filter(ressource => ressource.relationships.field_etape.data?.id === etapeId);
    const ressourcesRelatedPromises = ressourcesRelatedToEtape.map(ressource => getRessourceItemCard(ressource));

    return await Promise.all(ressourcesRelatedPromises);
}